/* Fonctions de traitement d'images
 *
 * Projet: Traitement d'images
 * Maitrise informatique 2003
 * Vouriot Thierry
 */

#include "ti.h"

extern char *traitement;

#define MAX(X,Y) ((X) < (Y)) ? (Y) : (X)
#define MIN(X,Y) ((X) < (Y)) ? (X) : (Y)
int min4(int a, int b, int c, int d)
{	
	if(a<=b && a<=c && a<=d) return a;
	else if(b<=a && b<=c && b<=d) return b;
	else if(c<=a && c<=b && c<= d) return c;
	else return d;
}

/* Retourne l'image */
void symetrie_verticale(unsigned char *bm, int h, int w, int np) 
{	
	unsigned char *tmp = NULL;
	int i;
	tmp = (unsigned char*) malloc(np * h * w * sizeof(unsigned char));
	
	for (i = 0; i < h; i++)
		memcpy(tmp + np*i*w, bm + (h-1)*w*np - i*np*w, w*np*sizeof(unsigned char));

	memcpy((void*)bm, (void*)tmp, w*h*np*sizeof(unsigned char)); 
		
	free(tmp);

	printf("--> Symetrie verticale\n");
	strcpy(traitement, "Symetrie verticale");
}

/* Negatif */
void negatif(unsigned char *bm, int h, int w, int np) 
{
	int i;
	int size = h*w*np;
	for (i = 0; i < size; i++)
		bm[i] = 255-bm[i];
	printf("--> Negatif\n");
	strcpy(traitement, "Negatif");
}

/* Niveau de gris */
void niveau_gris(unsigned char *bm, int h, int w, int np) 
{
	int i,tmp;
	int size = w*h*np;
	if (np == 3)
	for (i = 0; i < size; i+=np) {
		tmp = (int)(bm[i]*0.707+bm[i+1]*0.202+bm[i+2]*0.091);
		bm[i] = tmp;
		bm[i+1] = tmp;
		bm[i+2] = tmp;
	}
	printf("--> Niveau gris\n");
	strcpy(traitement, "Niveau gris");
}

/* Garde les couleurs R, V ou B demandees de l'image */
void couleurs(unsigned char *bm, int h, int w, int np, RGB c) 
{
	int i;
	int size = h*w*np;
	if (np == 3)
	for (i = 0; i < size; i+=np) {
		if (!c.r) bm[i] = 0;
		if (!c.g) bm[i+1] = 0;
		if (!c.b) bm[i+2] = 0;
	}
	printf("--> Selection de couleurs RGB: %d %d %d\n",c.r,c.g,c.b);
	strcpy(traitement, "Selection de couleurs");
}

/* Pour toutes les 3 composantes RVB de tous les pixels de l'image
 * on applique un masque pour ne garder que certains bits
 */
void select_bits(unsigned char *bm, int h, int w, int np, int masque) 
{	
	int i;
	int size = h*w*np;
	for (i = 0; i < size; i++) {
		bm[i] = bm[i] & masque;			
	}
	printf("--> Selection de bits\n");
	strcpy(traitement, "Selection de bits");
}

/* Egalisation d'histogramme 1 */
void egal_histo_1(unsigned char *bm, int h, int w, int np) 
{
	int i,D,tmp,ng;
	int size = h*w*np;
	int histo[256];
	int trans[256];
	
	for (i=0; i<256; i++) {
		histo[i] = 0;
	}
	for (i=0; i<size; i+=np) {
		ng = (int)(bm[i]*0.707+bm[i+1]*0.202+bm[i+2]*0.091);
		histo[bm[i]]++;
	}
	D = (h*w)/255;
	tmp = histo[0];
	trans[0] = 0;
	for (i=1; i<256; i++) {
		tmp += histo[i];
		trans[i] = tmp/D;	
	}
	for (i=0; i<size; i++) {
		bm[i] = trans[bm[i]];
	}
	printf("--> Egalisation histogramme 1\n");
	strcpy(traitement, "Egal. histo. 1");
}

/* Egalisation d'histogramme 2 */
void egal_histo_2(unsigned char *bm, int h, int w, int np) 
{
	int i,D,tmp,ng;
	int size = h*w*np;
	int histo[256];
	int trans[256];
	
	for (i=0; i<256; i++) {
		histo[i] = 0;
	}
	for (i=0; i<size; i+=3) {
		ng = (int)(bm[i]*0.707+bm[i+1]*0.202+bm[i+2]*0.091);
		histo[ng]++;
	}
	
	D = (h*w)/255;
	tmp = histo[0];
	trans[0] = 0;
	for (i=1; i<256; i++) {
		tmp += histo[i];
		trans[i] = tmp/(D*i);
		
	}
	for (i=0; i<size; i+=3) {
		ng = (int)(bm[i]*0.707+bm[i+1]*0.202+bm[i+2]*0.091);
		bm[i] = trans[ng] * bm[i];
		bm[i+1] = trans[ng] * bm[i+1];
		bm[i+2] = trans[ng] * bm[i+2];
	}
	printf("--> Egalisation histogramme 2\n");
	strcpy(traitement, "Egal. histo. 2");
}

/* Etirement */
void etirement(unsigned char* bm, int w, int h, int np)
{	int* bitmap = malloc(h*w*sizeof(int));
	int tmp;
	int taille = w*h*np, i, z=0,max = 0, min = 255;
	float transfert[256];

	for(i=0;i<taille;i+=3)
	{	bitmap[z] = (int)(bm[i]*0.707 + bm[i+1]*0.202 + bm[i+2]*0.091);
		if(bitmap[z]>max) max = bitmap[z];
		if(bitmap[z]<min) min = bitmap[z];
		z++;
	}

	for(i=0;i<256;i++)
	{
		tmp = (max-min) == 0 ? 1 : max-min ;
		transfert[i] = (float)(((i-min)*255)/(tmp));
	}

	z=0;

	for(i=0;i<taille;i+=3)
	{	
		tmp = (int)((float)bm[i]*transfert[bitmap[z]]/bitmap[z]);
		if(tmp>255) bm[i] = 255;
		else if(tmp<0) bm[i] = 0;
		else bm[i] = tmp;


		tmp = (int)((float)bm[i+1]*transfert[bitmap[z]]/bitmap[z]);
		if(tmp>255) bm[i+1] = 255;
		else if(tmp<0) bm[i+1] = 0;
		else bm[i+1] = tmp;


		tmp = (int)((float)bm[i+2]*transfert[bitmap[z]]/bitmap[z]);
		if(tmp>255) bm[i+2] = 255;
		else if(tmp<0) bm[i+2] = 0;
		else bm[i+2] = tmp;

		z++;
	}
	free(bitmap);

	printf("--> Etirement\n");
	strcpy(traitement, "Etirement");
}

/* Etirement interactif */
void etirement_interactif(unsigned char* bm, int w, int h, int np, int *coord)
{	int* bitmap = malloc(h*w*sizeof(int));
	int tmp;
	int taille = w*h*np, i, z=0,max, min;
	float transfert[256];

	for(i=0;i<taille;i+=3)
	{	bitmap[z] = (int)(bm[i]*0.707 + bm[i+1]*0.202 + bm[i+2]*0.091);
		z++;
	}

	min = bitmap[coord[0]*coord[1]];
	max = bitmap[coord[2]*coord[3]];

	for(i=0;i<256;i++)
	{
		tmp = (max-min) == 0 ? 1 : max-min ;
		transfert[i] = (float)(((i-min)*255)/(tmp));
	}

	z=0;

	for(i=0;i<taille;i+=3)
	{	tmp = (int)((float)bm[i]*transfert[bitmap[z]]/bitmap[z]);
		if(tmp>255) bm[i] = 255;
		else if(tmp<0) bm[i] = 0;
		else bm[i] = tmp;


		tmp = (int)((float)bm[i+1]*transfert[bitmap[z]]/bitmap[z]);
		if(tmp>255) bm[i+1] = 255;
		else if(tmp<0) bm[i+1] = 0;
		else bm[i+1] = tmp;


		tmp = (int)((float)bm[i+2]*transfert[bitmap[z]]/bitmap[z]);
		if(tmp>255) bm[i+2] = 255;
		else if(tmp<0) bm[i+2] = 0;
		else bm[i+2] = tmp;

		z++;
	}
	free(bitmap);

	printf("--> Etirement interactif (%d %d)\n", min, max);
	strcpy(traitement, "Etirement interactif");
}

/* Seuillage fixe */
void seuillage_fixe(unsigned char *bm, int h, int w, int np, int seuil) 
{
	int i,j;
	unsigned char ng;
	for(i=0; i<(h*w*np); i=i+np)
	{
		if (np==3) ng = (int)(bm[i]*0.707+bm[i+1]*0.202+bm[i+2]*0.091);
		else ng=bm[i];
		if (ng>seuil) ng=255; 
		else ng=0;
		for(j=0;j<np;j++) bm[i+j]=ng;
	}

	printf("--> Seuillage fixe\n");
	strcpy(traitement, "Seuillage fixe");
}

/* Seuillage aleatoire */
void seuillage_al(unsigned char *bm, int h, int w, int np) 
{
	int i,j;
	int s;
	unsigned char  ng;
	for(i=0; i<(h*w*np); i=i+np)
	{
		if (np==3) ng = (int)(bm[i]*0.707+bm[i+1]*0.202+bm[i+2]*0.091);
		else ng=bm[i];
		s=rand()%256;
		if (ng> s ) ng=255; 
		else ng=0;
		for(j=0;j<np;j++) bm[i+j]=ng;
	}

	printf("--> Seuillage aleatoire\n");
	strcpy(traitement, "Seuillage aleatoire");
}

/* Dent de scie */
void dent_scie(unsigned char * bm,int h, int w ,int np)
{
	int i,taille,tmp;
	taille=h*w*np;

	if(np==3)
	for(i=0; i<taille; i+=3)
	{
		tmp = (int)(bm[i]*0.707+bm[i+1]*0.202+bm[i+2]*0.091);
		bm[i] = (tmp*8)%256;
		bm[i+1] = (tmp*8)%256;
		bm[i+2] = (tmp*8)%256;
	}
	else
	for(i=0;i<taille;i++)
		bm[i] = (bm[i]*8)%256;

	printf("--> Dent de scie\n");
	strcpy(traitement, "Dent de scie");
}

/* 4 distance fond/point */
void distance_point4(unsigned char *bm, int h, int w, int np, RGB c, int *b) 
{
	int i, ptx, pty, x, y, d;
	float l;
	int size = h*w*np;
	
	pty = (h - b[1]);
	ptx = b[0]*np;

	l = (float)(abs(b[0] - b[2]) + abs(b[1] - b[3])) / (float)((float)(w*h)/5000.0f);
	if (l<1) l = 1;
	
	for (i = 0; i < size; i+=np) {
		y = i/(w*np);
		x = i%(w*np);
		// Calcul de la distance
		d = (int)((abs(x - ptx)/np)/l + abs(y - pty)/l);
		if (d > 255) d = 255;
		
		if (np == 1) bm[i] = d;
		else {
			if (c.r) bm[i] = d;
			if (c.g) bm[i+1] = d;
			if (c.b) bm[i+2] = d;
		}
	}
	printf("--> 4 distance (fond/point) (%d %d)\n", ptx, pty);
	strcpy(traitement, "4 distance (fond/point)");
}

/* 8 distance fond/point */
void distance_point8(unsigned char *bm, int h, int w, int np, RGB c, int *b) 
{	
	int i, ptx, pty, x, y, d;
	float l;
	int size = h*w*np;
	
	pty = (h - b[1]);
	ptx = b[0]*np;

	l = (float)(abs(b[0] - b[2]) + abs(b[1] - b[3])) / (float)((float)(w*h)/5000.0f);
	if (l<1) l = 1;
	
	for (i = 0; i < size; i+=np) {
		y = i/(w*np);
		x = i%(w*np);
		// Calcul de la distance
		d = (int)(MAX((abs(x - ptx)/np)/l, abs(y - pty)/l));
		if (d > 255) d = 255;
		if (np == 1) bm[i] = d;
		else {
			if (c.r) bm[i] = d;
			if (c.g) bm[i+1] = d;
			if (c.b) bm[i+2] = d;
		}
	}
	printf("--> 8 distance (fond/point) (%d %d)\n", ptx, pty);
	strcpy(traitement, "8 distance (fond/point)");
}

/* renvois la valeur d'un des 8 voisins si celui ci  une couleur suprieur  128 et diffrente de la couleur du pixel courant */
int colvoisin (unsigned char *bm,int h,int l, int i)
{	
	int col;
	col=bm[i*l+h];
	
	if (bm[(i-1)*l+h]>128 && bm[(i-1)*l+h]!=col)
		return bm[(i-1)*l+h];
	else if (bm[(i+1)*l+h]>128 && bm[(i+1)*l+h]!=col)
		return bm[(i+1)*l+h];
	else return col;
}

/* Remplissage par balayage */
void remplissage(unsigned char *bm, int h, int w, int np) 
{
	int i,j,k,color;
	int size= h*w*np;
	
	color=125;

	for (i = 0; i < h; i++)

		for (j=0;j< w;j++)
		{
			if(bm[i*w+j]>128)
			{
				k=j;
				while(bm[i*w+j]>128)
				{
					if (bm[(i-1)*w+j]>128)
					{
						color=bm[(i-1)*w+j];
						for(;k<j;k++)
							bm[k+(i)*w]=color;
					}
					bm[i*w+j]=color;
					j++;
				}
				//printf(" color %d\n",color);
				color+=25;
			}
		}

	// parcours de l'image pour galiser les valeurs
	for (i = 0; i < h; i++)

		for (j=0;j< w;j++)
		{	
			if(bm[i*w+j]>128)
			{
				color=colvoisin(bm,j,w,i);
				if (color != bm[i*w+j])
					for (k=0;k<size;k++)
						if (bm[k]==bm[i*w+j])
							bm[k]=color;
			}
		}

	printf("--> Remplissage\n");
	strcpy(traitement, "Remplissage");

}

/* Mediatrice de 2 points 4 distance */
void mediatrice4(unsigned char* bm, int w, int h, int np)
{	int coord[4], i, x1, y1, dist1, dist2;
	int taille = w*h, j = 0;

	/* recherche des coordonnees des deux points */
	for(i=0;i<taille;i++)
	{	if(bm[i] >100)
		{	coord[j] = i%w;
			coord[j+1] = i/w;
			j=2;
		}
	}

	/* recherche des points de la mediatrice et calcul des distances */
	for(i=0;i<taille;i++)
	{	x1 = i%w;
		y1 = i/w;
		dist1 = abs(x1 - coord[0]) + abs(y1 - coord[1]);
		dist2 = abs(x1 - coord[2]) + abs(y1 - coord[3]);

		if(dist1 == dist2)
		{	if(255 - dist1 < 0) bm[i] = 0;
			else bm[i] = 255 - dist1;
		}
	}

	printf("--> Mediatrice 4 distance\n");
	strcpy(traitement, "Mediatrice 4 distance");
}


/* Mediatrice de 2 points 8 distance */
void mediatrice8(unsigned char* bm, int w, int h, int np)
{	int coord[4], i, x1, y1, dist1, dist2;
	int taille = w*h, j = 0;

	/* recherche des coordonnees des deux points */
	for(i=0;i<taille;i++)
	{	if(bm[i] >100)
		{	coord[j] = i%w;
			coord[j+1] = i/w;
			j=2;
		}
	}

	/* recherche des points de la mediatrice et calcul des distances */
	for(i=0;i<taille;i++)
	{	x1 = i%w;
		y1 = i/w;
		dist1 = abs(x1 - coord[0]);
		dist2 = abs(x1 - coord[2]);

		/* recherche du max */
		if(dist1 < abs(y1 - coord[1])) dist1 = abs(y1 - coord[1]);
		if(dist2 < abs(y1 - coord[3])) dist2 = abs(y1 - coord[3]);

		if(dist1 == dist2)
		{	if(255 - dist1 < 0) bm[i] = 0;
			else bm[i] = 255 - dist1;
		}
	}

	printf("--> Mediatrice 8 distance\n");
	strcpy(traitement, "Mediatrice 8 distance");
}

/* 4 distance fond/figure */
void distance4_back_fig(unsigned char* bm, int w, int h, int np)
{	
	int taille = w*h, i, j=0, dist1, dist2;
	int* distance = malloc(w*h*sizeof(int));

	// Initialisation
	for(i=0;i<taille;i++)
	{	if(bm[i]>200)	distance[i]=0;
		else distance[i] = 255;
	}

	// Balayage 1
	// Premiere ligne
	for(i=1;i<w;i++)
	{	dist1 = distance[i-1]+1;
		if(distance[i]>dist1)	distance[i] = dist1;
	}
	// Reste de l'image
	for(i=1;i<h;i++)
	{	dist1 =distance[(i-1)*w]+1;
		if(distance[i*w]>dist1) distance[i*w] = dist1;

		for(j=1;j<w;j++)
		{	dist1 = distance[(i-1)*w+j]+1;
			dist2 = distance[i*w+j-1]+1;
			dist1 = MIN(dist1,dist2);

			if(distance[i*w+j]>dist1)
				distance[i*w+j] = dist1;
		}
	}

	// Balayage 2
	// Derniere ligne
	for(i=taille-2;i>=taille-w;i--)
	{	dist1 = distance[i+1]+1;
		if(distance[i]>dist1)	distance[i] = dist1;
	}
	// Reste de l'image
	for(i=h-2;i>=0;i--)
	{	dist1 =distance[(i+1)*w]+1;
		if(distance[i*w]>dist1) distance[i*w] = dist1;

		for(j=w-1;j>=0;j--)
		{	dist1 = distance[(i+1)*w+j]+1;
			dist2 = distance[i*w+j+1]+1;
			dist1 = MIN(dist1,dist2);

			if(distance[i*w+j]>dist1)
				distance[i*w+j] = dist1;
		}
	}

	// Copie du rsultat
	for(i=0;i<taille;i++)
		bm[i] = distance[i];

	free(distance);
	printf("--> 4 distance (fond/figure)\n");
	strcpy(traitement, "4 distance (fond/figure)");
}

/* 8 distance fond/figure */
void distance8_back_fig(unsigned char* bm, int w, int h, int np)
{	
	int taille = w*h, i, j=0, dist1, dist2,dist3,dist4,dist5;
	int* distance = malloc(w*h*sizeof(int));

	// Initialisation
	for(i=0;i<taille;i++)
	{	if(bm[i]>200)	distance[i]=0;
		else distance[i] = 255;
	}

	// Balayage 1
	// Premiere ligne
	for(i=1;i<w;i++)
	{	dist1 = distance[i-1]+3;
		if(distance[i]>dist1)	distance[i] = dist1;
	}
	// Reste de l'image
	for(i=1;i<h;i++)
	{	dist1 = distance[(i-1)*w]+3;
		dist2 = distance[(i-1)*w+1]+4;
		dist1 = MIN(dist1,dist2);

		if(distance[i*w] > dist1)
				distance[i*w] = dist1;

		for(j=1;j<w;j++)
		{	dist1 = distance[(i-1)*w+j]+3;
			dist2 = distance[i*w+j-1]+3;
			dist3 = distance[(i-1)*w + j-1]+4;
			dist4 = distance[(i-1)*w + j+1]+4;
			dist5 = min4(dist1,dist2,dist3,dist4);

			if(distance[i*w+j] > dist5)
				distance[i*w+j] = dist5;
		}
	}

	// Balayage 2
	// Derniere ligne
	for(i=taille-2;i>=taille-w;i--)
	{	dist1 = distance[i+1]+3;
		if(distance[i]>dist1)	distance[i] = dist1;
	}
	// Reste de l'image
	for(i=h-2;i>=0;i--)
	{	dist1 = distance[(i+1)*w]+3;
		dist2 = distance[(i+1)*w-1]+4;
		dist1 = MIN(dist1,dist2);

		if(distance[i*w] > dist1)
				distance[i*w] = dist1;

		for(j=w-1;j>=0;j--)
		{	dist1 = distance[(i+1)*w+j]+3;
			dist2 = distance[i*w+j+1]+4;
			dist3 = distance[(i+1)*w+j+1]+4;
			dist4 = distance[(i+1)*w+j-1]+4;
			dist5 = min4(dist1,dist2,dist3,dist4);

			if(distance[i*w+j] > dist5)
				distance[i*w+j] = dist5;
		}
	}

	// Copie du rsultat
	for(i=0;i<taille;i++)
		bm[i] = distance[i];

	free(distance);
	printf("--> 8 distance (fond/figure)\n");
	strcpy(traitement, "8 distance (fond/figure)");
}


/* 4 distance figure/fond */
void distance4_fig_back(unsigned char* bm, int w, int h, int np)
{	
	int taille = w*h, i, j=0, dist1, dist2;
	int* distance = malloc(w*h*sizeof(int));

	// Initialisation
	for(i=0;i<taille;i++)
	{	if(bm[i]>200)	distance[i]=255;
		else distance[i] = 0;
	}

	// Balayage 1
	// Premiere ligne
	for(i=1;i<w;i++)
	{	dist1 = distance[i-1]+1;
		if(distance[i]>dist1)	distance[i] = dist1;
	}
	// Reste de l'image
	for(i=1;i<h;i++)
	{	dist1 =distance[(i-1)*w]+1;
		if(distance[i]>dist1) distance[i*w] = dist1;

		for(j=1;j<w;j++)
		{	dist1 = distance[(i-1)*w+j]+1;
			dist2 = distance[i*w+j-1]+1;

			if(distance[i*w+j]>dist1)
			{	if(dist1>dist2)	distance[i*w+j] = dist2;
				else distance[i*w+j] = dist1;
			}
			else if(distance[i*w+j]>dist2)
					distance[i*w+j] = dist2;
		}
	}

	// Balayage 2
	// Derniere ligne
	for(i=taille-2;i>=taille-w;i--)
	{	dist1 = distance[i+1]+1;
		if(distance[i]>dist1)	distance[i] = dist1;
	}
	// Reste de l'image
	for(i=h-2;i>=0;i--)
	{	dist1 =distance[(i+1)*w]+1;
		if(distance[i]>dist1) distance[i*w] = dist1;

		for(j=w-1;j>=0;j--)
		{	dist1 = distance[(i+1)*w+j]+1;
			dist2 = distance[i*w+j+1]+1;

			if(distance[i*w+j]>dist1)
			{	if(dist1>dist2)	distance[i*w+j] = dist2;
				else distance[i*w+j] = dist1;
			}
			else if(distance[i*w+j]>dist2)
					distance[i*w+j] = dist2;
		}
	}

	// Copie du rsultat
	for(i=0;i<taille;i++)
		bm[i] = distance[i];

	free(distance);
	printf("--> 4 distance (figure/fond)\n");
	strcpy(traitement, "4 distance (figure/fond)");
}

/* 8 distance figure/fond */
void distance8_fig_back(unsigned char* bm, int w, int h, int np)
{	
	int taille = w*h, i, j=0, dist1, dist2,dist3,dist4,dist5;
	int* distance = malloc(w*h*sizeof(int));

	// Initialisation
	for(i=0;i<taille;i++)
	{	
		if(bm[i]>200)	distance[i]=255;
		else distance[i] = 0;
	}

	// Balayage 1
	// Premiere ligne
	for(i=1;i<w;i++)
	{	
		dist1 = distance[i-1]+3;
		if(distance[i]>dist1)	distance[i] = dist1;
	}
	// Reste de l'image
	for(i=1;i<h;i++)
	{	
		dist1 = distance[(i-1)*w]+3;
		dist2 = distance[(i-1)*w+1]+4;
		dist1 = MIN(dist1,dist2);

		if(distance[i*w] > dist1)
				distance[i*w] = dist1;

		for(j=1;j<w;j++)
		{	dist1 = distance[(i-1)*w+j]+3;
			dist2 = distance[i*w+j-1]+3;
			dist3 = distance[(i-1)*w + j-1]+4;
			dist4 = distance[(i-1)*w + j+1]+4;
			dist5 = min4(dist1,dist2,dist3,dist4);

			if(distance[i*w+j] > dist5)
				distance[i*w+j] = dist5;
		}
	}

	// Balayage 2
	// Derniere ligne
	for(i=taille-2;i>=taille-w;i--)
	{	dist1 = distance[i+1]+3;
		if(distance[i]>dist1)	distance[i] = dist1;
	}
	// Reste de l'image
	for(i=h-2;i>=0;i--)
	{	dist1 = distance[(i+1)*w]+3;
		dist2 = distance[(i+1)*w-1]+4;
		dist1 = MIN(dist1,dist2);

		if(distance[i*w] > dist1)
				distance[i*w] = dist1;

		for(j=w-1;j>=0;j--)
		{	dist1 = distance[(i+1)*w+j]+3;
			dist2 = distance[i*w+j+1]+4;
			dist3 = distance[(i+1)*w+j+1]+4;
			dist4 = distance[(i+1)*w+j-1]+4;
			dist5 = min4(dist1,dist2,dist3,dist4);

			if(distance[i*w+j] > dist5)
				distance[i*w+j] = dist5;
		}
	}

	// Copie du rsultat
	for(i=0;i<taille;i++)
		bm[i] = distance[i];

	free(distance);
	printf("--> 8 distance (figure/fond)\n");
	strcpy(traitement, "8 distance (figure/fond)");
}

/* Squelettisation M.A.T.*/
void squelet_mat(unsigned char* bm, int w, int h, int np)
{	
	int i, j;
	int taille = h*w*np;
	int* buffer = malloc(h*w*sizeof(int));

	distance4_fig_back(bm,w,h,np);

	for(i=0;i<taille;i++)
		buffer[i] = 0;

	for(i=w;i<taille-w;i++)
	{	if(bm[i] != 0)
		{	j = bm[i]+1;
			if(bm[i-1] == j || bm[i+1] == j || bm[i-w] == j
			|| bm[i+w] == j)
				buffer[i] = 0;
			else buffer[i] = bm[i];
		}
	}

	for(i=0;i<taille;i++)
		bm[i] = buffer[i];

	free(buffer);
	printf("--> Squelettisation M.A.T.\n");
	strcpy(traitement, "Squelettisation M.A.T.");
}

/* Verifie si un point est non isole et non terminal */
int non_term_isole(unsigned char* bm, int h, int w, int i, int j)
{	
	int tmp = 0;

	if(bm[j*h+i+1] > 10) tmp++;
	if(bm[j*h+i-1] > 10) tmp++;
	if(bm[(j-1)*h+i] > 10) tmp++;
	if(bm[(j+1)*h+i] > 10) tmp++;

	if(tmp>1) return 1;
	else return 0;
}

/* Nombre de Yokoi */
int yokoi(unsigned char* bm, int h, int w, int i, int j)
{	
	int x0 = 0, x1 = 0, x2 = 0, x3 = 0, x4 = 0, x5 = 0, x6 = 0, x7 = 0, res = 0;

	if(bm[(j-1)*h + i] > 10) x0 = 1;
	if(bm[(j-1)*h + i + 1] > 10) x1 = 1;
	if(bm[j*h + i + 1] > 10) x2 = 1;
	if(bm[(j+1)*h + i+1] > 10) x3 = 1;
	if(bm[(j+1)*h + i] > 10) x4 = 1;
	if(bm[(j+1)*h + i - 1] > 10) x5 = 1;
	if(bm[j*h + i - 1] > 10) x6 = 1;
	if(bm[(j-1)*h + i - 1] > 10) x7 = 1;

	res = x0*(1-x1*x2) + x2*(1-x3*x4) + x4*(1-x5*x6) + x6*(1-x7*x0);

	if(res == 1) return 1;
	else return 0;
}

/* Squelettisation de Rosenfeld */
void squelet_rosenfeld(unsigned char* bm, int w, int h, int np)
{	int i, j, modif = 1, taille = w*h;
	int* buffer = malloc(taille*sizeof(int));

	for(i=0;i<taille;i++)
		buffer[i] = bm[i];

	while(modif)
	{	modif = 0;

		// point nord
		for(j=1;j<h-1;j++)
			for(i=1;i<w-1;i++)
			{
				if(bm[j*h+i] > 10 && bm[(j-1)*h+i] < 10)
					if(non_term_isole(bm,h,w,i,j) && yokoi(bm,h,w,i,j))
					{	buffer[j*h + i] = 0;
						modif = 1;
					}
			}

		for(i=0;i<taille;i++)
			bm[i] = buffer[i];

		// point ouest
		for(j=1;j<h-1;j++)
			for(i=1;i<w-1;i++)
			{
				if(bm[j*h+i] > 10 && bm[j*h+i-1] < 10)
					if(non_term_isole(bm,h,w,i,j) && yokoi(bm,h,w,i,j))
					{	buffer[j*h + i] = 0;
						modif = 1;
					}
			}

		for(i=0;i<taille;i++)
			bm[i] = buffer[i];

		// point sud
		for(j=1;j<h-1;j++)
			for(i=1;i<w-1;i++)
			{
				if(bm[j*h+i] > 10 && bm[(j+1)*h+i] < 10)
					if(non_term_isole(bm,h,w,i,j) && yokoi(bm,h,w,i,j))
					{	buffer[j*h + i] = 0;
						modif = 1;
					}
			}

		for(i=0;i<taille;i++)
			bm[i] = buffer[i];

		// point est
		for(j=1;j<h-1;j++)
			for(i=1;i<w-1;i++)
			{
				if(bm[j*h+i] > 10 && bm[j*h+i+1] < 10)
					if(non_term_isole(bm,h,w,i,j) && yokoi(bm,h,w,i,j))
					{	buffer[j*h + i] = 0;
						modif = 1;
					}
			}

		for(i=0;i<taille;i++)
			bm[i] = buffer[i];
	}

	free(buffer);
	printf("--> Squelettisation Rosenfeld\n");
	strcpy(traitement, "Squelettisation Rosenfeld");
}

int connexit(unsigned char* bm, int h, int w, int i, int j)
{	int alt = 0;
	int color[8];
	int k = 0;

	if(bm[h*(j-1)+i] < 128) color[0] = 0;
	else color[0] = 1;
	if(bm[h*(j-1)+i+1] < 128) color[1] = 0;
	else color[1] = 1;
	if(bm[h*j+i+1] < 128) color[2] = 0;
	else color[2] = 1;
	if(bm[h*(j+1)+i+1] < 128) color[3] = 0;
	else color[3] = 1;
	if(bm[h*(j+1)+i] < 128) color[4] = 0;
	else color[4] = 1;
	if(bm[h*(j+1)+i-1] < 128) color[5] = 0;
	else color[5] = 1;
	if(bm[h*j+i-1] < 128) color[6] = 0;
	else color[6] = 1;
	if(bm[h*(j-1)+i-1] < 128) color[7] = 0;
	else color[7] = 1;
	for(k=0;k<7;k++)
		if(color[k] == 0 && color[k+1] == 1)   alt++;
	if(color[7] == 0 && color[0] == 1) alt++;

	return alt;
}

/* Squelettisation de Hilditch */
void squelet_hilditch(unsigned char* bm, int w, int h, int np)
{	
	int i, j, modif = 1, tmp;
	int taille = w*h;
	unsigned char* buffer = (unsigned char*)malloc(taille*sizeof(unsigned char));
	unsigned char* p1;
	unsigned char* p2;
	int swap = 0;
	
	p1 = bm;
	p2 = buffer;

	while(modif)
	{	modif = 0;

		for(j=1;j<h-1;j++)
			for(i=1;i<w-1;i++)
			{	p2[i+j*h] = 0;

				/*1ere cond - appartenance a la figure*/
				if(p1[j*h+i] > 128)
				{	tmp=0;

					/* test du nombre de voisin != du fond */
					if(p1[(j-1)*h+i] > 128) tmp++;
					if(p1[(j+1)*h+i] > 128) tmp++;
					if(p1[j*h+i+1] > 128) tmp++;
					if(p1[j*h+i-1] > 128) tmp++;
					if(p1[(j+1)*h+i-1] > 128) tmp++;
					if(p1[(j-1)*h+i-1] > 128) tmp++;
					if(p1[(j+1)*h+i+1] > 128) tmp++;
					if(p1[(j-1)*h+i+1] > 128) tmp++;

					if(tmp<2 || tmp>6)
					{	p2[i+j*h] = 255;
						continue;
					}

					if(connexit(p1,h,w,i,j) != 1)
					{	p2[i+j*h] = 255;
						continue;
					}

					if(p1[(j-1)*h+i] > 128 && p1[j*h+i-1] > 128 && p1[j*h+i+1] > 128)
						if(connexit(p1,h,w,i,j-1) == 1)
						{	p2[i+j*h] = 255;
							continue;
						}

					if(p1[(j-1)*h+i] > 128 && p1[(j+1)*h+i] > 128 && p1[j*h+i+1] > 128)
						if(connexit(p1,h,w,i+1,j) == 1)
						{	p2[i+j*h] = 255;
							continue;
						}

					p2[i+j*h] = 0;
					modif = 1;
				}
			}
			
			/* pour le switch de buffer */
			if(swap == 0)
			{	p1 = buffer;
				p2 = bm;
				swap = 1;
			}
			else
			{	p2 = buffer;
				p1 = bm;
				swap = 0;
			}
	}

	free(buffer);
}
